@@ -14,7 +14,7 @@
  * limitations under the License.
  */
 
-#include "src/binary-reader-objdump.h"
+#include "wabt/binary-reader-objdump.h"
 
 #include <algorithm>
 #include <cassert>
@@ -27,10 +27,10 @@
 #include <strings.h>
 #endif
 
-#include "src/binary-reader-nop.h"
-#include "src/filenames.h"
-#include "src/literal.h"
-#include "src/string-util.h"
+#include "wabt/binary-reader-nop.h"
+#include "wabt/filenames.h"
+#include "wabt/literal.h"
+#include "wabt/string-util.h"
 
 namespace wabt {
 
@@ -535,6 +535,10 @@ class BinaryReaderObjdumpDisassemble : public BinaryReaderObjdumpBase {
   Result OnOpcodeUint32Uint32Uint32(uint32_t value,
                                     uint32_t value2,
                                     uint32_t value3) override;
+  Result OnOpcodeUint32Uint32Uint32Uint32(uint32_t value,
+                                          uint32_t value2,
+                                          uint32_t value3,
+                                          uint32_t value4) override;
   Result OnOpcodeUint64(uint64_t value) override;
   Result OnOpcodeF32(uint32_t value) override;
   Result OnOpcodeF64(uint64_t value) override;
@@ -829,6 +833,18 @@ Result BinaryReaderObjdumpDisassemble::OnOpcodeUint32Uint32Uint32(
   return Result::Ok;
 }
 
+Result BinaryReaderObjdumpDisassemble::OnOpcodeUint32Uint32Uint32Uint32(
+    uint32_t value,
+    uint32_t value2,
+    uint32_t value3,
+    uint32_t value4) {
+  if (!in_function_body) {
+    return Result::Ok;
+  }
+  LogOpcode("%u %u %u %u", value, value2, value3, value4);
+  return Result::Ok;
+}
+
 Result BinaryReaderObjdumpDisassemble::OnOpcodeUint64(uint64_t value) {
   if (!in_function_body) {
     return Result::Ok;
