@@ -24,21 +24,21 @@
 #include <string>
 #include <vector>
 
-#include "src/binary-reader.h"
-#include "src/cast.h"
-#include "src/common.h"
-#include "src/error-formatter.h"
-#include "src/feature.h"
-#include "src/interp/binary-reader-interp.h"
-#include "src/interp/interp-util.h"
-#include "src/interp/interp.h"
-#include "src/literal.h"
-#include "src/option-parser.h"
-#include "src/stream.h"
-#include "src/string-util.h"
-#include "src/validator.h"
-#include "src/wast-lexer.h"
-#include "src/wast-parser.h"
+#include "wabt/binary-reader.h"
+#include "wabt/cast.h"
+#include "wabt/common.h"
+#include "wabt/error-formatter.h"
+#include "wabt/feature.h"
+#include "wabt/interp/binary-reader-interp.h"
+#include "wabt/interp/interp-util.h"
+#include "wabt/interp/interp.h"
+#include "wabt/literal.h"
+#include "wabt/option-parser.h"
+#include "wabt/stream.h"
+#include "wabt/string-util.h"
+#include "wabt/validator.h"
+#include "wabt/wast-lexer.h"
+#include "wabt/wast-parser.h"
 
 using namespace wabt;
 using namespace wabt::interp;
@@ -198,41 +198,39 @@ ExpectedValue GetLane(const ExpectedValue& ev, int lane) {
 
   v128 vec = ev.value.value.Get<v128>();
 
-  for (int lane = 0; lane < lane_count; ++lane) {
-    switch (ev.lane_type) {
-      case Type::I8:
-        result.nan[0] = ExpectedNan::None;
-        result.value.value.Set<u32>(vec.u8(lane));
-        break;
+  switch (ev.lane_type) {
+    case Type::I8:
+      result.nan[0] = ExpectedNan::None;
+      result.value.value.Set<u32>(vec.u8(lane));
+      break;
 
-      case Type::I16:
-        result.nan[0] = ExpectedNan::None;
-        result.value.value.Set<u32>(vec.u16(lane));
-        break;
+    case Type::I16:
+      result.nan[0] = ExpectedNan::None;
+      result.value.value.Set<u32>(vec.u16(lane));
+      break;
 
-      case Type::I32:
-        result.nan[0] = ExpectedNan::None;
-        result.value.value.Set<u32>(vec.u32(lane));
-        break;
+    case Type::I32:
+      result.nan[0] = ExpectedNan::None;
+      result.value.value.Set<u32>(vec.u32(lane));
+      break;
 
-      case Type::I64:
-        result.nan[0] = ExpectedNan::None;
-        result.value.value.Set<u64>(vec.u64(lane));
-        break;
+    case Type::I64:
+      result.nan[0] = ExpectedNan::None;
+      result.value.value.Set<u64>(vec.u64(lane));
+      break;
 
-      case Type::F32:
-        result.nan[0] = ev.nan[lane];
-        result.value.value.Set<f32>(Bitcast<f32>(vec.f32_bits(lane)));
-        break;
+    case Type::F32:
+      result.nan[0] = ev.nan[lane];
+      result.value.value.Set<f32>(Bitcast<f32>(vec.f32_bits(lane)));
+      break;
 
-      case Type::F64:
-        result.nan[0] = ev.nan[lane];
-        result.value.value.Set<f64>(Bitcast<f64>(vec.f64_bits(lane)));
-        break;
+    case Type::F64:
+      result.nan[0] = ev.nan[lane];
+      result.value.value.Set<f64>(Bitcast<f64>(vec.f64_bits(lane)));
+      break;
 
-      default:
-        WABT_UNREACHABLE;
-    }
+    default:
+      WABT_UNREACHABLE;
   }
   return result;
 }
@@ -247,35 +245,33 @@ TypedValue GetLane(const TypedValue& tv, Type lane_type, int lane) {
 
   v128 vec = tv.value.Get<v128>();
 
-  for (int lane = 0; lane < lane_count; ++lane) {
-    switch (lane_type) {
-      case Type::I8:
-        result.value.Set<u32>(vec.u8(lane));
-        break;
+  switch (lane_type) {
+    case Type::I8:
+      result.value.Set<u32>(vec.u8(lane));
+      break;
 
-      case Type::I16:
-        result.value.Set<u32>(vec.u16(lane));
-        break;
+    case Type::I16:
+      result.value.Set<u32>(vec.u16(lane));
+      break;
 
-      case Type::I32:
-        result.value.Set<u32>(vec.u32(lane));
-        break;
+    case Type::I32:
+      result.value.Set<u32>(vec.u32(lane));
+      break;
 
-      case Type::I64:
-        result.value.Set<u64>(vec.u64(lane));
-        break;
+    case Type::I64:
+      result.value.Set<u64>(vec.u64(lane));
+      break;
 
-      case Type::F32:
-        result.value.Set<f32>(Bitcast<f32>(vec.f32_bits(lane)));
-        break;
+    case Type::F32:
+      result.value.Set<f32>(Bitcast<f32>(vec.f32_bits(lane)));
+      break;
 
-      case Type::F64:
-        result.value.Set<f64>(Bitcast<f64>(vec.f64_bits(lane)));
-        break;
+    case Type::F64:
+      result.value.Set<f64>(Bitcast<f64>(vec.f64_bits(lane)));
+      break;
 
-      default:
-        WABT_UNREACHABLE;
-    }
+    default:
+      WABT_UNREACHABLE;
   }
   return result;
 }
@@ -1274,6 +1270,7 @@ wabt::Result CommandRunner::Run(const Script& script) {
   for (const CommandPtr& command : script.commands) {
     switch (command->type) {
       case CommandType::Module:
+      case CommandType::ScriptModule:
         TallyCommand(OnModuleCommand(cast<ModuleCommand>(command.get())));
         break;
 
