@@ -14,20 +14,20 @@
  * limitations under the License.
  */
 
-#include "src/validator.h"
+#include "wabt/validator.h"
 
 #include <cassert>
 #include <cinttypes>
 #include <cstdarg>
 #include <cstdio>
 
-#include "config.h"
+#include "wabt/config.h"
 
-#include "src/binary-reader.h"
-#include "src/cast.h"
-#include "src/expr-visitor.h"
-#include "src/ir.h"
-#include "src/shared-validator.h"
+#include "wabt/binary-reader.h"
+#include "wabt/cast.h"
+#include "wabt/expr-visitor.h"
+#include "wabt/ir.h"
+#include "wabt/shared-validator.h"
 
 namespace wabt {
 
@@ -280,7 +280,7 @@ Result Validator::OnCallRefExpr(CallRefExpr* expr) {
   Index function_type_index;
   result_ |= validator_.OnCallRef(expr->loc, &function_type_index);
   if (Succeeded(result_)) {
-    expr->function_type_index = Var{function_type_index};
+    expr->function_type_index = Var{function_type_index, expr->loc};
     return Result::Ok;
   }
 
@@ -537,7 +537,7 @@ Result Validator::OnRethrowExpr(RethrowExpr* expr) {
 }
 
 Result Validator::OnAtomicWaitExpr(AtomicWaitExpr* expr) {
-  result_ |= validator_.OnAtomicWait(expr->loc, expr->opcode,
+  result_ |= validator_.OnAtomicWait(expr->loc, expr->opcode, expr->memidx,
                                      expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
@@ -548,32 +548,33 @@ Result Validator::OnAtomicFenceExpr(AtomicFenceExpr* expr) {
 }
 
 Result Validator::OnAtomicNotifyExpr(AtomicNotifyExpr* expr) {
-  result_ |= validator_.OnAtomicNotify(expr->loc, expr->opcode,
+  result_ |= validator_.OnAtomicNotify(expr->loc, expr->opcode, expr->memidx,
                                        expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
 Result Validator::OnAtomicLoadExpr(AtomicLoadExpr* expr) {
-  result_ |= validator_.OnAtomicLoad(expr->loc, expr->opcode,
+  result_ |= validator_.OnAtomicLoad(expr->loc, expr->opcode, expr->memidx,
                                      expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
 Result Validator::OnAtomicStoreExpr(AtomicStoreExpr* expr) {
-  result_ |= validator_.OnAtomicStore(expr->loc, expr->opcode,
+  result_ |= validator_.OnAtomicStore(expr->loc, expr->opcode, expr->memidx,
                                       expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
 Result Validator::OnAtomicRmwExpr(AtomicRmwExpr* expr) {
-  result_ |= validator_.OnAtomicRmw(expr->loc, expr->opcode,
+  result_ |= validator_.OnAtomicRmw(expr->loc, expr->opcode, expr->memidx,
                                     expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
 Result Validator::OnAtomicRmwCmpxchgExpr(AtomicRmwCmpxchgExpr* expr) {
-  result_ |= validator_.OnAtomicRmwCmpxchg(
-      expr->loc, expr->opcode, expr->opcode.GetAlignment(expr->align));
+  result_ |=
+      validator_.OnAtomicRmwCmpxchg(expr->loc, expr->opcode, expr->memidx,
+                                    expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
@@ -588,14 +589,14 @@ Result Validator::OnSimdLaneOpExpr(SimdLaneOpExpr* expr) {
 }
 
 Result Validator::OnSimdLoadLaneExpr(SimdLoadLaneExpr* expr) {
-  result_ |= validator_.OnSimdLoadLane(expr->loc, expr->opcode,
+  result_ |= validator_.OnSimdLoadLane(expr->loc, expr->opcode, expr->memidx,
                                        expr->opcode.GetAlignment(expr->align),
                                        expr->val);
   return Result::Ok;
 }
 
 Result Validator::OnSimdStoreLaneExpr(SimdStoreLaneExpr* expr) {
-  result_ |= validator_.OnSimdStoreLane(expr->loc, expr->opcode,
+  result_ |= validator_.OnSimdStoreLane(expr->loc, expr->opcode, expr->memidx,
                                         expr->opcode.GetAlignment(expr->align),
                                         expr->val);
   return Result::Ok;
@@ -607,13 +608,13 @@ Result Validator::OnSimdShuffleOpExpr(SimdShuffleOpExpr* expr) {
 }
 
 Result Validator::OnLoadSplatExpr(LoadSplatExpr* expr) {
-  result_ |= validator_.OnLoadSplat(expr->loc, expr->opcode,
+  result_ |= validator_.OnLoadSplat(expr->loc, expr->opcode, expr->memidx,
                                     expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
 
 Result Validator::OnLoadZeroExpr(LoadZeroExpr* expr) {
-  result_ |= validator_.OnLoadZero(expr->loc, expr->opcode,
+  result_ |= validator_.OnLoadZero(expr->loc, expr->opcode, expr->memidx,
                                    expr->opcode.GetAlignment(expr->align));
   return Result::Ok;
 }
@@ -958,6 +959,13 @@ void ScriptValidator::CheckCommand(const Command* command) {
       break;
     }
 
+    case CommandType::ScriptModule: {
+      Validator module_validator(
+          errors_, &cast<ScriptModuleCommand>(command)->module, options_);
+      module_validator.CheckModule();
+      break;
+    }
+
     case CommandType::Action:
       // Ignore result type.
       CheckAction(cast<ActionCommand>(command)->action.get());
